package com.bpcl.lpg.core.service.impl;

import de.hybris.platform.servicelayer.exceptions.SystemException;
import org.apache.logging.log4j.LogManager;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycAccessTokenResponseData;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.net.URI;
import java.net.http.HttpRequest;
import java.net.http.HttpClient;
import java.security.spec.InvalidKeySpecException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.security.spec.KeySpec;
import java.security.KeyFactory;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.security.PrivateKey;
import java.security.Key;
import io.jsonwebtoken.SignatureAlgorithm;
import java.time.temporal.TemporalUnit;
import java.time.temporal.ChronoUnit;
import io.jsonwebtoken.Jwts;
import java.time.Instant;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycTransactionData;
import com.bpcl.lpg.facades.ekyc.data.BpclLastEkycTransactionDetails;
import java.util.Date;
import java.security.SecureRandom;
import com.bpcl.lpg.webservices.request.dto.LpgOperatorWhitelistCodeWsDTO;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import com.bpcl.lpg.core.enums.StaffRole;
import de.hybris.platform.b2b.model.B2BUnitModel;
import com.bpcl.lpg.facades.data.LoginRequestData;
import de.hybris.platform.servicelayer.exceptions.AmbiguousIdentifierException;
import de.hybris.platform.servicelayer.exceptions.ModelNotFoundException;
import java.text.MessageFormat;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.bpcl.lpg.facades.ekyc.data.OperatorRequestData;
import com.bpcl.lpg.facades.ekyc.data.OperatorResponseData;
import java.util.Optional;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.util.Objects;
import com.bpcl.lpg.webservices.request.dto.BpclEkycStatusRequestWsDTO;
import de.hybris.platform.core.model.user.UserModel;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.model.LPGDistributorStaffModel;
import java.net.http.HttpResponse;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycTransactionRequestDetails;
import java.util.List;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycTransactionRequestData;
import com.bpcl.lpg.core.model.BpclEkycConsentsModel;
import com.bpcl.core.model.BPCLCustomerModel;
import com.bpcl.core.model.BPCLB2BUnitModel;
import de.hybris.platform.b2b.services.B2BUnitService;
import com.bpcl.lpg.core.model.BpclEkycUpdateStatusModel;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycStatusResponseData;
import com.bpcl.lpg.core.model.BpclEkycTransactionModel;
import com.bpcl.lpg.facades.ekyc.data.BpclEkycTransactionResponseData;
import de.hybris.platform.converters.Populator;
import com.bpcl.lpg.core.stub.ekyc.BpclEkycTransactionRequestDataStub;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.user.UserService;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.lpg.core.dao.BpclEkycDao;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import org.apache.logging.log4j.Logger;
import com.bpcl.lpg.core.service.BpclEkycService;

public class BpclEkycServiceImpl implements BpclEkycService
{
    private static final Logger LOG;
    private static final String DEBUG_JSON_FORMATTER = "Input json :: {0}";
    private static final String DEBUG_BODY_MSG = "The request body being sent for transaction id is: ";
    private static final String DEBUG_BODY_MSG_FOR_OPERATOR = "The request body being sent for getting Operator details api is: ";
    private static final String ENABLE_STUB_FOR_EKYC_TRXN = "ekyc.trxn.stub";
    private static final String EKYC_TRANSACTION_PARTNER_CODE_CUSTOMER = "ekyc.transaction.partner.code.customer";
    private static final String EKYC_TRANSACTION_PARTNER_CODE_DELIVERY = "ekyc.transaction.partner.code.delivery";
    private static final String EKYC_TRANSACTION_EKYC_TYPE = "ekyc.transaction.ekyc.type";
    private static final String EKYC_TRANSACTION_API_URL = "ekyc.transaction.url";
    private static final String EKYC_TRANSACTION_CUSTOMER_API_NAME = "ekyc.transaction.customer.name";
    private static final String EKYC_TRANSACTION_PARTNER_API_NAME = "ekyc.transaction.partner.name";
    private static final String EKYC_TRANSACTION_URL_UNIQUE_REQ = "?uniqueApiReqNo=";
    public static final String EKYC_OPERATOR_DETAILS_API_URL = "ekyc.operator.details.url";
    public static final String EKYC_OPERATOR_DETAILS_API_NAME = "ekyc.operator.details.name";
    private static final String EPARICHAY_CLIENT_ID = "client_id=";
    private static final String EPARICHAY_CLIENT_SECRET = "&client_secret=";
    private static final String EPARICHAY_GRANT_TYPE = "&grant_type=";
    private static final String EPARICHAY_SCOPE = "&scope=";
    private static final String EPARICHAY_TOKEN_URL = "eparichay.token.url";
    private static final String EPARICHAY_TOKEN_CLIENT_ID = "eparichay.token.client.id";
    private static final String EPARICHAY_TOKEN_CLIENT_SECRET = "eparichay.token.client.secret";
    private static final String EPARICHAY_TOKEN_GRANT_TYPE = "eparichay.token.grant.type";
    private static final String EPARICHAY_TOKEN_SCOPE = "eparichay.token.scope";
    private static final String IS_EKYC_PERFORMED = "1";
    private static final String EKYC_TYPE_FINGER = "2";
    private static final String EKYC_TYPE_IRIS = "3";
    private static final String EKYC_TYPE_FACE = "4";
    private static final String IS_EKYC_TYPE_PERFORMED = "0";
    private static final String EKYC_CONFIG = "stub.lpg.ekyc.operator.response";
    private static final String EKYC_PRIVATE_KEY = "generate.token.ekyc.rsa.privatekey";
    private static final String ISSUER = "generate.token.ekyc.issuer";
    private static final String AUDIENCE = "generate.token.ekyc.audience";
    private static final String UPDATE_EKYC_STATUS_API_URL = "update.ekyc.status.url";
    private static final String UPDATE_EKYC_STATUS_URL_UNIQUE_REQ = "?uniqueApiReqNo=";
    private static final String EKYC_JWT_TOKEN_EXP_TIME = "ekyc.transaction.jwt.expiry.time";
    private static final String EKYC_UPDATE_SUCCESS_MESSAGE = "eKYC verification successful";
    private static final int EKYC_UPDATE_SUCCESS_CODE = 200;
    private static final String EKYC_UPDATE_FAILURE_MESSAGE = "Successfully Updated Status as eKYC Failed";
    private static final int EKYC_UPDATE_FAILURE_CODE = 202;
    private static final String DEBUG_BODY_MSG_FOR_UPDATE_STATUS = "The request body being sent for update status is: ";
    private static final int RETURN_CODE_711 = 711;
    private static final String EKYC_SAVE_REQUEST_BODY = "ekyc.save.request";
    private static final String CORRELATION_HEADER_KEY = "X-BPCL-Correlation-Id";
    @Resource
    private KeyGenerator jwtKeyGenerator;
    @Resource
    private BpclEkycDao bpclEkycDao;
    @Resource
    private ConfigurationService configurationService;
    @Resource
    private BpclHttpClientService bpclHttpClientService;
    @Resource
    private LPGCredentialsService lpgCredentialsService;
    @Resource
    private UserService userService;
    @Resource
    private ModelService modelService;
    @Resource
    private LpgEncryptionService lpgEncryptionService;
    @Resource
    private BpclEkycTransactionRequestDataStub bpclEkycTransactionRequestDataStub;
    @Resource(name = "bpclEkycTransactionReversePopulator")
    private Populator<BpclEkycTransactionResponseData, BpclEkycTransactionModel> bpclEkycTransactionReversePopulator;
    @Resource(name = "bpclEkycUpdateStatusReversePopulator")
    private Populator<BpclEkycStatusResponseData, BpclEkycUpdateStatusModel> bpclEkycUpdateStatusReversePopulator;
    @Resource(name = "b2bUnitService")
    private B2BUnitService<BPCLB2BUnitModel, BPCLCustomerModel> b2bUnitService;
    @Resource
    private KeyGenerator ekycKeyGenerator;
    @Resource
    private KeyGenerator ekycOperatorKeyGenerator;
    @Resource
    private LpgCustomerServiceImpl lpgCustomerService;
    @Resource(name = "bpclEkycConsentTransactionRequestPopulator")
    private Populator<BpclEkycConsentsModel, BpclEkycTransactionRequestData> bpclEkycConsentTransactionRequestPopulator;
    
    public List<BpclEkycConsentsModel> getAllBpclEkycConsents(final boolean isHelloBpcl) {
        final String partnerCode = (this.userService.getCurrentUser() instanceof BPCLCustomerModel) ? this.configurationService.getConfiguration().getString("ekyc.transaction.partner.code.customer") : this.configurationService.getConfiguration().getString("ekyc.transaction.partner.code.delivery");
        return this.bpclEkycDao.getAllBpclEkycConsents(partnerCode);
    }
    
    public boolean isOTPAuthSuccess(final String customerId) {
        return this.bpclEkycDao.isOTPAuthSuccess(customerId);
    }
    
    public BpclEkycTransactionResponseData getBpclEkycTransaction(final BpclEkycTransactionRequestDetails request) throws Exception {
        BpclEkycTransactionRequestData bpclEkycTransactionRequestData = new BpclEkycTransactionRequestData();
        final String partnerCode = (this.userService.getCurrentUser() instanceof BPCLCustomerModel) ? this.configurationService.getConfiguration().getString("ekyc.transaction.partner.code.customer") : this.configurationService.getConfiguration().getString("ekyc.transaction.partner.code.delivery");
        final String EKYC_TRANSACTION_API_NAME = (this.userService.getCurrentUser() instanceof BPCLCustomerModel) ? this.configurationService.getConfiguration().getString("ekyc.transaction.customer.name") : this.configurationService.getConfiguration().getString("ekyc.transaction.partner.name");
        final BpclEkycConsentsModel consentByConsentId = this.bpclEkycDao.getConsentByLangCode(request.getLanguageCode(), partnerCode);
        if (this.configurationService.getConfiguration().getBoolean("ekyc.trxn.stub")) {
            this.bpclEkycTransactionRequestDataStub.populateStubbedTrxnRequestDataForEkyc(bpclEkycTransactionRequestData);
        }
        else {
            bpclEkycTransactionRequestData = this.populateReqDataForEkycTrxn(request, consentByConsentId);
        }
        final StringBuilder ekycTransUrl = new StringBuilder(this.configurationService.getConfiguration().getString("ekyc.transaction.url")).append("?uniqueApiReqNo=").append(bpclEkycTransactionRequestData.getUniqueApiReqNo());
        final HttpResponse<String> response = this.bpclHttpClientService.executePostMethod(ekycTransUrl.toString(), this.getRequestJson(bpclEkycTransactionRequestData), EKYC_TRANSACTION_API_NAME);
        final BpclEkycTransactionResponseData bpclEkycTransactionResponseData = this.convertToResponse(BpclEkycTransactionResponseData.class, response.body());
        this.populateTknAndTrxnModel(bpclEkycTransactionRequestData, bpclEkycTransactionResponseData, response);
        this.populateResponseMessageForError(bpclEkycTransactionResponseData);
        return bpclEkycTransactionResponseData;
    }
    
    public BpclEkycTransactionModel getEkycTrxnByTrxnId(final String eKycTransactionId) {
        return this.bpclEkycDao.getEkycTrxnByTrxnId(eKycTransactionId);
    }
    
    private void populateResponseMessageForError(BpclEkycTransactionResponseData bpclEkycTransactionResponseData) {
        if (null == bpclEkycTransactionResponseData) {
            bpclEkycTransactionResponseData = new BpclEkycTransactionResponseData();
            bpclEkycTransactionResponseData.setReturnCode(400);
            bpclEkycTransactionResponseData.setReturnMsg("Error in submitting request. Please try again later.");
        }
        else if (727 == bpclEkycTransactionResponseData.getReturnCode()) {
            bpclEkycTransactionResponseData.setReturnCode(727);
            bpclEkycTransactionResponseData.setReturnMsg(bpclEkycTransactionResponseData.getReturnMsg());
        }
        else if (200 != bpclEkycTransactionResponseData.getReturnCode() && 201 != bpclEkycTransactionResponseData.getReturnCode()) {
            bpclEkycTransactionResponseData.setReturnCode(404);
            final UserModel currentUser = this.userService.getCurrentUser();
            if (currentUser instanceof final LPGDistributorStaffModel lpgDistributorStaffModel) {
                bpclEkycTransactionResponseData.setReturnMsg(bpclEkycTransactionResponseData.getReturnMsg());
            }
            else {
                bpclEkycTransactionResponseData.setReturnMsg("eKYC could not be done. Please contact your LPG distributor.");
            }
        }
        else if (null != bpclEkycTransactionResponseData.getData() && (StringUtils.isBlank(bpclEkycTransactionResponseData.getData().getEKycReferenceKey()) || (StringUtils.isNotBlank(bpclEkycTransactionResponseData.getData().getEKycReferenceKey()) && "0".equalsIgnoreCase(bpclEkycTransactionResponseData.getData().getEKycReferenceKey())))) {
            bpclEkycTransactionResponseData.setReturnCode(404);
            bpclEkycTransactionResponseData.setReturnMsg("eKYC could not be done. Please contact your LPG distributor.");
        }
    }
    
    public BpclEkycStatusResponseData updateEKycStatus(final BpclEkycStatusRequestWsDTO requestData) throws JsonProcessingException {
        StringBuilder ekycUpdateEKycStatusUrl = new StringBuilder(this.configurationService.getConfiguration().getString("update.ekyc.status.url")).append("?uniqueApiReqNo=").append(requestData.getUniqueApiReqNo());
        final String EKYC_TRANSACTION_API_NAME = (this.userService.getCurrentUser() instanceof BPCLCustomerModel) ? this.configurationService.getConfiguration().getString("ekyc.transaction.customer.name") : this.configurationService.getConfiguration().getString("ekyc.transaction.partner.name");
        BpclEkycStatusResponseData bpclEkycStatusResponseData = null;
        HttpResponse<String> response = this.bpclHttpClientService.executePostMethod(ekycUpdateEKycStatusUrl.toString(), this.getEkycStatusRequestJson(requestData), EKYC_TRANSACTION_API_NAME);
        boolean isMessageUpdateRequired = false;
        if (Objects.isNull(response)) {
            requestData.setUniqueApiReqNo(this.ekycKeyGenerator.generate().toString());
            ekycUpdateEKycStatusUrl = new StringBuilder(this.configurationService.getConfiguration().getString("update.ekyc.status.url")).append("?uniqueApiReqNo=").append(requestData.getUniqueApiReqNo());
            isMessageUpdateRequired = true;
            response = this.bpclHttpClientService.executePostMethod(ekycUpdateEKycStatusUrl.toString(), this.getEkycStatusRequestJson(requestData), EKYC_TRANSACTION_API_NAME);
        }
        bpclEkycStatusResponseData = this.convertToResponse(BpclEkycStatusResponseData.class, response.body());
        if (isMessageUpdateRequired) {
            if (response.statusCode() == 200) {
                bpclEkycStatusResponseData.setReturnCode(200);
                bpclEkycStatusResponseData.setReturnMsg("eKYC verification successful");
            }
            else if (response.statusCode() == 711) {
                bpclEkycStatusResponseData.setReturnCode(202);
                bpclEkycStatusResponseData.setReturnMsg("Successfully Updated Status as eKYC Failed");
            }
        }
        this.populateEkycUpdateStatusToTrxn(requestData, bpclEkycStatusResponseData, response);
        return bpclEkycStatusResponseData;
    }
    
    private void populateEkycUpdateStatusToTrxn(final BpclEkycStatusRequestWsDTO requestData, final BpclEkycStatusResponseData bpclEkycStatusResponseData, final HttpResponse<String> response) throws JsonProcessingException {
        try {
            final BpclEkycUpdateStatusModel ekycUpdateStatusModel = (BpclEkycUpdateStatusModel)this.modelService.create((Class)BpclEkycUpdateStatusModel.class);
            this.bpclEkycUpdateStatusReversePopulator.populate((Object)bpclEkycStatusResponseData, (Object)ekycUpdateStatusModel);
            final boolean saveRequestBody = this.configurationService.getConfiguration().getBoolean("ekyc.save.request", false);
            if (saveRequestBody) {
                ekycUpdateStatusModel.setRequestBody(this.getEkycStatusRequestJson(requestData));
            }
            if (null != response.headers()) {
                final Optional<String> correlationId = response.headers().firstValue("X-BPCL-Correlation-Id");
                if (correlationId.isPresent()) {
                    ekycUpdateStatusModel.setCorrelationId((String)correlationId.get());
                }
            }
            if (StringUtils.isNotBlank(requestData.getCode())) {
                ekycUpdateStatusModel.setReceivedCode(requestData.getCode());
            }
            if (StringUtils.isNotBlank(requestData.getMessage())) {
                ekycUpdateStatusModel.setReceivedMsg(requestData.getMessage());
            }
            if (StringUtils.isNotBlank(requestData.getEkycTransactionId())) {
                final BpclEkycTransactionModel ekycTrxnByTrxnId = this.bpclEkycDao.getEkycTrxnByTrxnId(requestData.getEkycTransactionId());
                if (null != ekycTrxnByTrxnId) {
                    ekycUpdateStatusModel.setEkycTransaction(ekycTrxnByTrxnId);
                    this.modelService.saveAll(new Object[] { ekycUpdateStatusModel, ekycTrxnByTrxnId });
                }
            }
        }
        catch (final Exception ex) {
            BpclEkycServiceImpl.LOG.error("An exception occured while populating ekyc status on trxn: ".concat(ex.getMessage()));
        }
    }
    
    public OperatorResponseData getOperatorDetailsFromLpg(final String contactNumber) throws JsonProcessingException {
        final OperatorRequestData operatorRequestData = new OperatorRequestData();
        if (this.configurationService.getConfiguration().getBoolean("stub.lpg.ekyc.operator.response", false)) {
            this.bpclEkycTransactionRequestDataStub.populateStubResponseForOperatorRequest(operatorRequestData);
        }
        else {
            this.populateOperatorRequestData(contactNumber, operatorRequestData);
        }
        final StringBuilder operatorDetailsUrl = new StringBuilder(this.configurationService.getConfiguration().getString("ekyc.operator.details.url")).append("?uniqueApiReqNo=").append(operatorRequestData.getUniqueApiReqNo());
        if (BpclEkycServiceImpl.LOG.isDebugEnabled()) {
            BpclEkycServiceImpl.LOG.debug("The request body being sent for getting Operator details api is: ".concat(this.getRequestJsonForOperator(operatorRequestData)));
        }
        return this.convertToResponse(OperatorResponseData.class, this.bpclHttpClientService.executePostMethod(operatorDetailsUrl.toString(), this.getRequestJsonForOperator(operatorRequestData), this.configurationService.getConfiguration().getString("ekyc.operator.details.name")).body());
    }
    
    private String getRequestJsonForOperator(final OperatorRequestData operatorRequestData) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(operatorRequestData);
        if (BpclEkycServiceImpl.LOG.isDebugEnabled()) {
            BpclEkycServiceImpl.LOG.debug(MessageFormat.format("Input json :: {0}", json));
        }
        return objectMapper.writeValueAsString(operatorRequestData);
    }
    
    private void populateOperatorRequestData(final String contactNumber, final OperatorRequestData operatorRequestData) {
        operatorRequestData.setPartnerCode(this.configurationService.getConfiguration().getString("ekyc.transaction.partner.code.delivery"));
        operatorRequestData.setPartnerTransId(this.ekycOperatorKeyGenerator.generate().toString());
        operatorRequestData.setUniqueApiReqNo(this.ekycKeyGenerator.generate().toString());
        operatorRequestData.setMobileNumber(contactNumber);
        final String hashValue = this.getHashKeyValueRequest(operatorRequestData);
        operatorRequestData.setHashValue(hashValue);
    }
    
    private String getHashKeyValueRequest(final OperatorRequestData operatorRequestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = this.configurationService.getConfiguration().getString("ekyc.operator.details.name");
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        hashMessage.append(this.getEmptyIfBlank(operatorRequestData.getPartnerCode()));
        hashMessage.append(this.getEmptyIfBlank(operatorRequestData.getPartnerTransId()));
        hashMessage.append(this.getEmptyIfBlank(operatorRequestData.getUniqueApiReqNo()));
        hashMessage.append(this.getEmptyIfBlank(operatorRequestData.getMobileNumber()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    public boolean doesOperatorStaffExist(final String contactNumber) {
        try {
            final LPGDistributorStaffModel bpclCustomerModel = this.getBpclOperatorByUid(contactNumber);
            return Objects.nonNull(bpclCustomerModel);
        }
        catch (final ModelNotFoundException | AmbiguousIdentifierException exception) {
            return false;
        }
    }
    
    public LPGDistributorStaffModel getBpclOperatorByUid(final String customerId) {
        return this.bpclEkycDao.findOperatorByCustomerId(customerId);
    }
    
    public void createOperator(final OperatorResponseData operatorResponseData, final LoginRequestData loginRequestData, final Boolean isOperatorUpdate, final LPGDistributorStaffModel existingOperatorModel) throws ParseException {
        if (Objects.nonNull(operatorResponseData)) {
            LPGDistributorStaffModel operatorModel = null;
            if (!isOperatorUpdate) {
                operatorModel = (LPGDistributorStaffModel)this.modelService.create((Class)LPGDistributorStaffModel.class);
                operatorModel.setUid(loginRequestData.getUserId());
                operatorModel.setDefaultB2BUnit((B2BUnitModel)this.b2bUnitService.getUnitForUid("Dummy"));
                operatorModel.setEmail("dummy@test.com");
                operatorModel.setStaffRole(StaffRole.OPERATORS);
            }
            else {
                operatorModel = existingOperatorModel;
            }
            if (Objects.nonNull(operatorResponseData.getData())) {
                if (StringUtils.isNotBlank(operatorResponseData.getData().getDistributorCode())) {
                    operatorModel.setDistributorId(operatorResponseData.getData().getDistributorCode());
                }
                if (StringUtils.isNotBlank(operatorResponseData.getData().getDistributorName())) {
                    operatorModel.setDistributorName(operatorResponseData.getData().getDistributorName());
                }
                if (Objects.nonNull(operatorResponseData.getData().getOperatorDetails())) {
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getOperatorStatus())) {
                        final Boolean status = operatorResponseData.getData().getOperatorDetails().getOperatorStatus().equalsIgnoreCase("1") ? Boolean.TRUE : Boolean.FALSE;
                        operatorModel.setActiveStatus(status);
                    }
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getOperatorValidTill())) {
                        operatorModel.setOperatorValidTill(new SimpleDateFormat("yyyy-MM-dd").parse(operatorResponseData.getData().getOperatorDetails().getOperatorValidTill()));
                    }
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getOperatorMobileNumber())) {
                        operatorModel.setMobileNumber(operatorResponseData.getData().getOperatorDetails().getOperatorMobileNumber());
                    }
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getOperatorId())) {
                        operatorModel.setStaffId(operatorResponseData.getData().getOperatorDetails().getOperatorId());
                    }
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getOperatorName())) {
                        operatorModel.setStaffName(operatorResponseData.getData().getOperatorDetails().getOperatorName());
                    }
                    if (StringUtils.isNotBlank(operatorResponseData.getData().getOperatorDetails().getWhitelistingflag())) {
                        final Boolean whitelistingFlag = operatorResponseData.getData().getOperatorDetails().getWhitelistingflag().equalsIgnoreCase("1") ? Boolean.TRUE : Boolean.FALSE;
                        operatorModel.setWhitelistingFlag(whitelistingFlag);
                    }
                }
            }
            this.modelService.save((Object)operatorModel);
        }
    }
    
    public String generateWhitelistCode(final LpgOperatorWhitelistCodeWsDTO requestWsDTO) {
        final String mobileNumber = "op".concat(requestWsDTO.getMobileNumber());
        final LPGDistributorStaffModel operatorModel = this.getBpclOperatorByUid(mobileNumber);
        if (operatorModel.getWhitelistingFlag()) {
            throw new RuntimeException("Login not allowed. Please contact distributor");
        }
        final SecureRandom random = new SecureRandom();
        final String code = String.valueOf(100000 + random.nextInt(900000));
        operatorModel.setMobileDeviceId(requestWsDTO.getMobileDeviceId());
        operatorModel.setMobileModel(requestWsDTO.getMobileModel());
        operatorModel.setMobileOSVersion(requestWsDTO.getMobileOSVersion());
        operatorModel.setWhitelistingCode(code);
        operatorModel.setCodeCreationtime(new Date());
        this.modelService.save((Object)operatorModel);
        BpclEkycServiceImpl.LOG.info("Operator Whitelist device details : {} : {} : {} ", (Object)mobileNumber, (Object)requestWsDTO.getMobileDeviceId(), (Object)code);
        return code;
    }
    
    private void populateTknAndTrxnModel(final BpclEkycTransactionRequestData bpclEkycTransactionRequestData, final BpclEkycTransactionResponseData bpclEkycTransactionResponseData, final HttpResponse<String> response) throws Exception {
        if (null != bpclEkycTransactionResponseData) {
            final BpclEkycTransactionModel bpclEkycTransactionModel = (BpclEkycTransactionModel)this.modelService.create((Class)BpclEkycTransactionModel.class);
            this.bpclEkycTransactionReversePopulator.populate((Object)bpclEkycTransactionResponseData, (Object)bpclEkycTransactionModel);
            final boolean saveRequestBody = this.configurationService.getConfiguration().getBoolean("ekyc.save.request", false);
            if (saveRequestBody) {
                bpclEkycTransactionModel.setRequestBody(this.getRequestJson(bpclEkycTransactionRequestData));
            }
            if (null != response.headers()) {
                final Optional<String> correlationId = response.headers().firstValue("X-BPCL-Correlation-Id");
                if (correlationId.isPresent()) {
                    bpclEkycTransactionModel.setCorrelationId((String)correlationId.get());
                }
            }
            this.modelService.save((Object)bpclEkycTransactionModel);
            if (200 == bpclEkycTransactionResponseData.getReturnCode() || 201 == bpclEkycTransactionResponseData.getReturnCode()) {
                if (null != bpclEkycTransactionResponseData.getData()) {
                    final BpclEkycTransactionData ekycTransactionResponseData = bpclEkycTransactionResponseData.getData();
                    if (StringUtils.isNotBlank(ekycTransactionResponseData.getEKycReferenceKey())) {
                        ekycTransactionResponseData.setReferenceKeyVal(Long.parseLong(ekycTransactionResponseData.getEKycReferenceKey()));
                    }
                    if (StringUtils.isNotBlank(ekycTransactionResponseData.getEKycTransactionId())) {
                        ekycTransactionResponseData.setTransactionIdVal(Long.parseLong(ekycTransactionResponseData.getEKycTransactionId()));
                    }
                }
                final UserModel currentUser = this.userService.getCurrentUser();
                if (currentUser instanceof final LPGDistributorStaffModel lpgDistributorStaffModel) {
                    final String distCode = lpgDistributorStaffModel.getStaffId();
                    bpclEkycTransactionResponseData.setAtk(this.generateJwtToken(distCode));
                    bpclEkycTransactionResponseData.setEtk(this.getBearerToken());
                }
                else if (null != bpclEkycTransactionResponseData.getData()) {
                    if (StringUtils.isNotBlank(bpclEkycTransactionResponseData.getData().getLpgId())) {
                        bpclEkycTransactionResponseData.setAtk(this.generateJwtToken(bpclEkycTransactionResponseData.getData().getLpgId()));
                    }
                    bpclEkycTransactionResponseData.setEtk(this.getBearerToken());
                }
            }
            if (null != bpclEkycTransactionResponseData.getData() && (!"1".equalsIgnoreCase(bpclEkycTransactionResponseData.getData().getIseKycPerformed()) || (null != bpclEkycTransactionResponseData.getData().getLasteKycTransactionDetails() && !"2".equalsIgnoreCase(bpclEkycTransactionResponseData.getData().getLasteKycTransactionDetails().getEKycType()) && !"3".equalsIgnoreCase(bpclEkycTransactionResponseData.getData().getLasteKycTransactionDetails().getEKycType()) && !"4".equalsIgnoreCase(bpclEkycTransactionResponseData.getData().getLasteKycTransactionDetails().getEKycType())))) {
                final BpclEkycTransactionData responseData = bpclEkycTransactionResponseData.getData();
                responseData.setIseKycPerformed("0");
                responseData.setLasteKycTransactionDetails((BpclLastEkycTransactionDetails)null);
            }
        }
    }
    
    private BpclEkycTransactionRequestData populateReqDataForEkycTrxn(final BpclEkycTransactionRequestDetails request, final BpclEkycConsentsModel consentByConsentId) {
        final BpclEkycTransactionRequestData bpclEkycTransactionRequestData = new BpclEkycTransactionRequestData();
        final UserModel currentUser = this.userService.getCurrentUser();
        if (currentUser instanceof final BPCLCustomerModel bpclCustomerModel) {
            bpclEkycTransactionRequestData.setPartnerCode(consentByConsentId.getPartnerCode());
            bpclEkycTransactionRequestData.setAppId(consentByConsentId.getAppID());
            if (StringUtils.isNotBlank(bpclCustomerModel.getUid())) {
                bpclEkycTransactionRequestData.setMobileNumber(bpclCustomerModel.getUid());
            }
            bpclEkycTransactionRequestData.setRequestedBy(request.getLpgId());
        }
        else {
            bpclEkycTransactionRequestData.setAppId(consentByConsentId.getAppID());
            bpclEkycTransactionRequestData.setPartnerCode(consentByConsentId.getPartnerCode());
            if (StringUtils.isNotBlank(request.getMobileNumber())) {
                bpclEkycTransactionRequestData.setMobileNumber(request.getMobileNumber());
            }
            if (StringUtils.isNotBlank(request.getLpgId())) {
                bpclEkycTransactionRequestData.setLpgId(request.getLpgId());
            }
            if (StringUtils.isNotBlank(request.getConsumerNumber()) && StringUtils.isNotBlank(request.getDistCode())) {
                bpclEkycTransactionRequestData.setConsumerNumber(request.getConsumerNumber());
                bpclEkycTransactionRequestData.setDistCode(request.getDistCode());
            }
            final BpclEkycTransactionRequestData bpclEkycTransactionRequestData2 = bpclEkycTransactionRequestData;
            final UserModel currentUser2 = this.userService.getCurrentUser();
            String staffId;
            if (currentUser2 instanceof final LPGDistributorStaffModel lpgDistributorStaffModel) {
                staffId = lpgDistributorStaffModel.getStaffId();
            }
            else {
                staffId = null;
            }
            bpclEkycTransactionRequestData2.setRequestedBy(staffId);
        }
        bpclEkycTransactionRequestData.setUniqueApiReqNo(this.ekycKeyGenerator.generate().toString());
        bpclEkycTransactionRequestData.setIsAudioPlayed(request.getIsAudioPlayed());
        bpclEkycTransactionRequestData.setEKycType(this.configurationService.getConfiguration().getString("ekyc.transaction.ekyc.type"));
        this.bpclEkycConsentTransactionRequestPopulator.populate((Object)consentByConsentId, (Object)bpclEkycTransactionRequestData);
        bpclEkycTransactionRequestData.setHashValue(this.getHashKeyValueForEkycTrxnRequest(bpclEkycTransactionRequestData));
        return bpclEkycTransactionRequestData;
    }
    
    public String generateJwtToken(final String userId) throws Exception {
        final Instant now = Instant.now();
        final String issuer = this.configurationService.getConfiguration().getString("generate.token.ekyc.issuer");
        final String audience = this.configurationService.getConfiguration().getString("generate.token.ekyc.audience");
        try {
            final PrivateKey privateKey = this.getPemPrivateKey();
            return Jwts.builder().setId(this.jwtKeyGenerator.generate().toString()).setSubject(userId).setIssuer(issuer).setAudience(audience).setNotBefore(Date.from(now)).setIssuedAt(Date.from(now)).setExpiration(Date.from(now.plus((long)this.configurationService.getConfiguration().getInt("ekyc.transaction.jwt.expiry.time", 5), (TemporalUnit)ChronoUnit.MINUTES))).signWith(SignatureAlgorithm.RS256, (Key)privateKey).compact();
        }
        catch (final Exception e) {
            BpclEkycServiceImpl.LOG.error("Exception while generating JWT token ", (Throwable)e);
            throw new Exception(e.getMessage());
        }
    }
    
    private PrivateKey getPemPrivateKey() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException {
        final String privateKeyFilename = this.configurationService.getConfiguration().getString("generate.token.ekyc.rsa.privatekey");
        final File privateKeyFile = new File(privateKeyFilename);
        final FileInputStream fileInputStream = new FileInputStream(privateKeyFile);
        final DataInputStream dataInputStream = new DataInputStream(fileInputStream);
        final byte[] keyBytes = fileInputStream.readAllBytes();
        dataInputStream.close();
        final String temp = new String(keyBytes);
        final String privateKeyString = temp.replace("-----BEGIN RSA PRIVATE KEY-----", "").replaceAll(System.lineSeparator(), "").replace("-----END RSA PRIVATE KEY-----", "");
        final PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKeyString));
        final KeyFactory kf = KeyFactory.getInstance("RSA");
        return kf.generatePrivate(keySpec);
    }
    
    public String getBearerToken() throws IOException, InterruptedException {
        final HttpClient client = HttpClient.newHttpClient();
        final HttpRequest request = HttpRequest.newBuilder().uri(URI.create(this.configurationService.getConfiguration().getString("eparichay.token.url"))).headers("Content-Type", "application/x-www-form-urlencoded").POST(HttpRequest.BodyPublishers.ofString("client_id=" + URLEncoder.encode(this.configurationService.getConfiguration().getString("eparichay.token.client.id"), StandardCharsets.UTF_8) + "&client_secret=" + URLEncoder.encode(this.configurationService.getConfiguration().getString("eparichay.token.client.secret"), StandardCharsets.UTF_8) + "&grant_type=" + URLEncoder.encode(this.configurationService.getConfiguration().getString("eparichay.token.grant.type"), StandardCharsets.UTF_8) + "&scope=" + URLEncoder.encode(this.configurationService.getConfiguration().getString("eparichay.token.scope"), StandardCharsets.UTF_8))).build();
        final HttpResponse<String> httpResponse = client.send(request, HttpResponse.BodyHandlers.ofString());
        if (null != httpResponse) {
            final BpclEkycAccessTokenResponseData bpclEkycAccessTokenResponseData = this.convertToResponse(BpclEkycAccessTokenResponseData.class, httpResponse.body());
            return bpclEkycAccessTokenResponseData.getAccess_token();
        }
        return "";
    }
    
    private <T> T convertToResponse(final Class<T> responseObject, final String responseBody) throws JsonProcessingException {
        final ObjectMapper mapper = new ObjectMapper();
        mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.NONE);
        mapper.setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        mapper.configure(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT, true);
        return mapper.readValue(responseBody, responseObject);
    }
    
    private String getRequestJson(final BpclEkycTransactionRequestData bpclEkycTransactionRequestData) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(bpclEkycTransactionRequestData);
        return objectMapper.writeValueAsString(bpclEkycTransactionRequestData);
    }
    
    private String getEkycStatusRequestJson(final BpclEkycStatusRequestWsDTO bpclEkycStatusRequestWsDTO) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(bpclEkycStatusRequestWsDTO);
        return objectMapper.writeValueAsString(bpclEkycStatusRequestWsDTO);
    }
    
    private String getHashKeyValueForEkycTrxnRequest(final BpclEkycTransactionRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = (this.userService.getCurrentUser() instanceof BPCLCustomerModel) ? this.configurationService.getConfiguration().getString("ekyc.transaction.customer.name") : this.configurationService.getConfiguration().getString("ekyc.transaction.partner.name");
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        hashMessage.append(this.getEmptyIfBlank(requestData.getPartnerCode())).append(this.getEmptyIfBlank(requestData.getPartnerTransId())).append(this.getEmptyIfBlank(requestData.getUniqueApiReqNo())).append(this.getEmptyIfBlank(requestData.getMobileNumber())).append(this.getEmptyIfBlank(requestData.getLpgId())).append(this.getEmptyIfBlank(requestData.getDistCode())).append(this.getEmptyIfBlank(requestData.getConsumerNumber())).append(this.getEmptyIfBlank(requestData.getAppId())).append(this.getEmptyIfBlank(requestData.getLanguageCode())).append(this.getEmptyIfBlank(requestData.getLanguageName())).append(this.getEmptyIfBlank(requestData.getConsentId())).append(this.getEmptyIfBlank(requestData.getConsentText())).append(this.getEmptyIfBlank(requestData.getIsAudioPlayed())).append(this.getEmptyIfBlank(requestData.getEKycType())).append(this.getEmptyIfBlank(requestData.getRequestedBy())).append(this.getEmptyIfBlank(secret));
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private String getEmptyIfBlank(final String stringToCheck) {
        return StringUtils.isBlank(stringToCheck) ? "" : stringToCheck;
    }
    
    public KeyGenerator getKeyGenerator() {
        return this.ekycKeyGenerator;
    }
    
    public void setKeyGenerator(final KeyGenerator keyGenerator) {
        this.ekycKeyGenerator = keyGenerator;
    }
    
    static {
        LOG = LogManager.getLogger((Class)BpclEkycServiceImpl.class);
    }
}
